@@ -48,8 +48,6 @@
 #include "gnome-keyring-md5.h"
 #include "wireless-dialog.h"
 
-#define PREF_SUPPRESS_WIRELESS_NEWORKS_AVAILABLE    APPLET_PREFS_PATH "/suppress-wireless-networks-available"
-
 #define ACTIVE_AP_TAG "active-ap"
 
 static void wireless_dialog_response_cb (GtkDialog *dialog, gint response, gpointer user_data);
@@ -117,6 +115,8 @@ nma_menu_add_create_network_item (GtkWidget *menu, NMApplet *applet)
 {
 	GtkWidget *menu_item;
 	GtkWidget *label;
+	GError *error = NULL;
+	gboolean disabled;
 
 	menu_item = gtk_menu_item_new ();
 	label = gtk_label_new_with_mnemonic (_("Create _New Wireless Network..."));
@@ -125,6 +125,14 @@ nma_menu_add_create_network_item (GtkWidget *menu, NMApplet *applet)
 	gtk_widget_show_all (menu_item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
 	g_signal_connect (menu_item, "activate", G_CALLBACK (new_network_activate_cb), applet);
+
+	/* FIXME: should really use PolicyKit and NM permissions here instead
+	 * using using GConf mandatory settings.  But this works for now.
+	 */
+	disabled = gconf_client_get_bool (applet->gconf_client, PREF_DISABLE_WIFI_CREATE, &error);
+	if (error || disabled)
+		gtk_widget_set_sensitive (GTK_WIDGET (menu_item), FALSE);
+	g_clear_error (&error);
 }
 
 
@@ -357,8 +365,15 @@ wireless_new_auto_connection (NMDevice *device,
 	dev_caps = nm_device_wifi_get_capabilities (NM_DEVICE_WIFI (device));
 	s_wireless_sec = get_security_for_ap (info->ap, dev_caps, &supported, &s_8021x);
 	if (!supported) {
+		g_warning ("Unsupported AP configuration: dev_caps 0x%X, ap_flags 0x%X, "
+		           "wpa_flags 0x%X, rsn_flags 0x%x, mode %d",
+		           dev_caps,
+		           nm_access_point_get_flags (info->ap),
+		           nm_access_point_get_wpa_flags (info->ap),
+		           nm_access_point_get_rsn_flags (info->ap),
+		           nm_access_point_get_mode (info->ap));
 		g_object_unref (s_wireless);
-		goto done;
+		return FALSE;
 	} else if (s_wireless_sec)
 		g_object_set (s_wireless, NM_SETTING_WIRELESS_SEC, NM_SETTING_WIRELESS_SECURITY_SETTING_NAME, NULL);
 
@@ -388,7 +403,6 @@ wireless_new_auto_connection (NMDevice *device,
 
 	nm_connection_add_setting (connection, NM_SETTING (s_con));
 
-done:
 	(*callback) (connection, TRUE, FALSE, callback_data);
 	return TRUE;
 }
@@ -893,7 +907,7 @@ wifi_available_dont_show_cb (NotifyNotification *notify,
 		return;
 
 	gconf_client_set_bool (applet->gconf_client,
-	                       PREF_SUPPRESS_WIRELESS_NEWORKS_AVAILABLE,
+	                       PREF_SUPPRESS_WIRELESS_NETWORKS_AVAILABLE,
 	                       TRUE,
 	                       NULL);
 }
@@ -996,7 +1010,7 @@ queue_avail_access_point_notification (NMDevice *device)
 		return;
 
 	if (gconf_client_get_bool (data->applet->gconf_client,
-	                           PREF_SUPPRESS_WIRELESS_NEWORKS_AVAILABLE,
+	                           PREF_SUPPRESS_WIRELESS_NETWORKS_AVAILABLE,
 	                           NULL))
 		return;
 
@@ -1308,6 +1322,7 @@ wireless_dialog_response_cb (GtkDialog *foo,
 	NMDevice *device = NULL;
 	NMAccessPoint *ap = NULL;
 	NMAGConfConnection *gconf_connection;
+	const char *service = NM_DBUS_SERVICE_USER_SETTINGS;
 
 	if (response != GTK_RESPONSE_OK)
 		goto done;
@@ -1332,6 +1347,12 @@ wireless_dialog_response_cb (GtkDialog *foo,
 	g_assert (connection);
 	g_assert (device);
 
+	/* If it's a system connection we just need to tell NM to activate it */
+	if (nm_connection_get_scope (connection) == NM_CONNECTION_SCOPE_SYSTEM) {
+		service = NM_DBUS_SERVICE_SYSTEM_SETTINGS;
+		goto activate;
+	}
+
 	gconf_connection = nma_gconf_settings_get_by_connection (applet->gconf_settings, connection);
 	if (gconf_connection) {
 		/* Not a new or system connection, save the updated settings to GConf */
@@ -1411,8 +1432,9 @@ wireless_dialog_response_cb (GtkDialog *foo,
 		}
 	}
 
+activate:
 	nm_client_activate_connection (applet->nm_client,
-	                               NM_DBUS_SERVICE_USER_SETTINGS,
+	                               service,
 	                               nm_connection_get_path (connection),
 	                               device,
 	                               ap ? nm_object_get_path (NM_OBJECT (ap)) : NULL,
@@ -1454,6 +1476,7 @@ add_one_setting (GHashTable *settings,
                  GError **error)
 {
 	GHashTable *secrets;
+	GError *tmp_error = NULL;
 
 	g_return_val_if_fail (settings != NULL, FALSE);
 	g_return_val_if_fail (connection != NULL, FALSE);
@@ -1461,7 +1484,15 @@ add_one_setting (GHashTable *settings,
 	g_return_val_if_fail (error != NULL, FALSE);
 	g_return_val_if_fail (*error == NULL, FALSE);
 
-	utils_fill_connection_certs (connection);
+	if (!utils_fill_connection_certs (NM_CONNECTION (connection), &tmp_error)) {
+		g_set_error (error, NM_SETTINGS_ERROR, NM_SETTINGS_ERROR_INTERNAL_ERROR,
+					 "%s.%d (%s): failed to read connection certificates: (%d) %s.",
+					 __FILE__, __LINE__, __func__,
+					 tmp_error ? tmp_error->code : -1,
+					 tmp_error && tmp_error->message ? tmp_error->message : "(unknown)");
+		return FALSE;
+	}
+
 	secrets = nm_setting_to_hash (setting);
 	utils_clear_filled_connection_certs (connection);
 
