@@ -70,6 +70,7 @@
 #include "utils.h"
 #include "gconf-helpers.h"
 
+#define NOTIFY_CAPS_ACTIONS_KEY "actions"
 
 G_DEFINE_TYPE(NMApplet, nma, G_TYPE_OBJECT)
 
@@ -106,20 +107,20 @@ applet_get_best_activating_connection (NMApplet *applet, NMDevice **device)
 		}
 
 		if (NM_IS_DEVICE_WIFI (best_dev)) {
-			if (NM_IS_DEVICE_ETHERNET (candidate)) {
+			if (NM_IS_DEVICE_ETHERNET (candidate_dev)) {
 				best_dev = candidate_dev;
 				best = candidate;
 			}
 		} else if (NM_IS_CDMA_DEVICE (best_dev)) {
-			if (   NM_IS_DEVICE_ETHERNET (candidate)
-			    || NM_IS_DEVICE_WIFI (candidate)) {
+			if (   NM_IS_DEVICE_ETHERNET (candidate_dev)
+			    || NM_IS_DEVICE_WIFI (candidate_dev)) {
 				best_dev = candidate_dev;
 				best = candidate;
 			}
 		} else if (NM_IS_GSM_DEVICE (best_dev)) {
-			if (   NM_IS_DEVICE_ETHERNET (candidate)
-			    || NM_IS_DEVICE_WIFI (candidate)
-			    || NM_IS_CDMA_DEVICE (candidate)) {
+			if (   NM_IS_DEVICE_ETHERNET (candidate_dev)
+			    || NM_IS_DEVICE_WIFI (candidate_dev)
+			    || NM_IS_CDMA_DEVICE (candidate_dev)) {
 				best_dev = candidate_dev;
 				best = candidate;
 			}
@@ -295,44 +296,62 @@ activate_connection_cb (gpointer user_data, const char *path, GError *error)
 	applet_schedule_update_icon (NM_APPLET (user_data));
 }
 
-void
-applet_menu_item_activate_helper (NMDevice *device,
-                                  NMConnection *connection,
-                                  const char *specific_object,
-                                  NMApplet *applet,
-                                  gpointer user_data)
+typedef struct {
+	NMApplet *applet;
+	NMDevice *device;
+	char *specific_object;
+	gpointer dclass_data;
+} AppletItemActivateInfo;
+
+static void
+applet_item_activate_info_destroy (AppletItemActivateInfo *info)
 {
-	NMAGConfConnection *exported;
+	g_return_if_fail (info != NULL);
+
+	if (info->device)
+		g_object_unref (info->device);
+	g_free (info->specific_object);
+	memset (info, 0, sizeof (AppletItemActivateInfo));
+	g_free (info);
+}
+
+static void
+applet_menu_item_activate_helper_part2 (NMConnection *connection,
+                                        gboolean auto_created,
+                                        gboolean canceled,
+                                        gpointer user_data)
+{
+	AppletItemActivateInfo *info = user_data;
 	const char *con_path;
 	gboolean is_system = FALSE;
 
-	g_return_if_fail (NM_IS_DEVICE (device));
+	if (canceled) {
+		applet_item_activate_info_destroy (info);
+		return;
+	}
 
-	if (connection) {
+	g_return_if_fail (connection != NULL);
+
+	if (!auto_created)
 		is_system = is_system_connection (connection);
-	} else {
-		NMADeviceClass *dclass = get_device_class (device, applet);
+	else {
+		NMAGConfConnection *exported;
 
-		/* If no connection was given, create a new default connection for this
-		 * device type.
-		 */
-		g_assert (dclass);
-		connection = dclass->new_auto_connection (device, applet, user_data);
-		if (!connection) {
-			nm_warning ("Couldn't create default connection.");
-			return;
-		}
-
-		exported = nma_gconf_settings_add_connection (applet->gconf_settings, connection);
+		exported = nma_gconf_settings_add_connection (info->applet->gconf_settings, connection);
 		if (!exported) {
+			NMADeviceClass *dclass = get_device_class (info->device, info->applet);
+
 			/* If the setting isn't valid, because it needs more authentication
 			 * or something, ask the user for it.
 			 */
 
+			g_assert (dclass);
 			nm_warning ("Invalid connection; asking for more information.");
 			if (dclass->get_more_info)
-				dclass->get_more_info (device, connection, applet, user_data);
+				dclass->get_more_info (info->device, connection, info->applet, info->dclass_data);
 			g_object_unref (connection);
+
+			applet_item_activate_info_destroy (info);
 			return;
 		}
 		g_object_unref (connection);
@@ -343,13 +362,52 @@ applet_menu_item_activate_helper (NMDevice *device,
 	g_assert (con_path);
 
 	/* Finally, tell NM to activate the connection */
-	nm_client_activate_connection (applet->nm_client,
+	nm_client_activate_connection (info->applet->nm_client,
 	                               is_system ? NM_DBUS_SERVICE_SYSTEM_SETTINGS : NM_DBUS_SERVICE_USER_SETTINGS,
 	                               con_path,
-	                               device,
-	                               specific_object,
+	                               info->device,
+	                               info->specific_object,
 	                               activate_connection_cb,
-	                               applet);
+	                               info->applet);
+	applet_item_activate_info_destroy (info);
+}
+
+void
+applet_menu_item_activate_helper (NMDevice *device,
+                                  NMConnection *connection,
+                                  const char *specific_object,
+                                  NMApplet *applet,
+                                  gpointer dclass_data)
+{
+	AppletItemActivateInfo *info;
+	NMADeviceClass *dclass;
+
+	g_return_if_fail (NM_IS_DEVICE (device));
+
+	info = g_malloc0 (sizeof (AppletItemActivateInfo));
+	info->applet = applet;
+	info->specific_object = g_strdup (specific_object);
+	info->device = g_object_ref (device);
+	info->dclass_data = dclass_data;
+
+	if (connection) {
+		applet_menu_item_activate_helper_part2 (connection, FALSE, FALSE, info);
+		return;
+	}
+
+	dclass = get_device_class (device, applet);
+
+	/* If no connection was passed in, ask the device class to create a new
+	 * default connection for this device type.  This could be a wizard,
+	 * and thus take a while.
+	 */
+	g_assert (dclass);
+	if (!dclass->new_auto_connection (device, dclass_data,
+	                                  applet_menu_item_activate_helper_part2,
+	                                  info)) {
+		nm_warning ("Couldn't create default connection.");
+		applet_item_activate_info_destroy (info);
+	}
 }
 
 static void
@@ -376,6 +434,7 @@ applet_do_notify (NMApplet *applet,
 {
 	NotifyNotification *notify;
 	GError *error = NULL;
+	char *escaped;
 
 	g_return_if_fail (applet != NULL);
 	g_return_if_fail (summary != NULL);
@@ -388,22 +447,27 @@ applet_do_notify (NMApplet *applet,
 
 	applet_clear_notify (applet);
 
-	notify = notify_notification_new (summary, message,
-	                                  icon ? icon : GTK_STOCK_NETWORK, NULL);
+	escaped = utils_escape_notify_message (message);
+	notify = notify_notification_new (summary,
+	                                  escaped,
+	                                  icon ? icon : GTK_STOCK_NETWORK,
+	                                  NULL);
+	g_free (escaped);
 	applet->notification = notify;
 
 	notify_notification_attach_to_status_icon (notify, applet->status_icon);
 	notify_notification_set_urgency (notify, urgency);
 	notify_notification_set_timeout (notify, NOTIFY_EXPIRES_DEFAULT);
 
-	if (action1) {
+	if (applet->notify_with_actions && action1) {
 		notify_notification_add_action (notify, action1, action1_label,
 		                                action1_cb, action1_user_data, NULL);
 	}
 
 	if (!notify_notification_show (notify, &error)) {
-		g_warning ("Failed to show notification: %s", error->message);
-		g_error_free (error);
+		g_warning ("Failed to show notification: %s",
+		           error && error->message ? error->message : "(unknown)");
+		g_clear_error (&error);
 	}
 }
 
@@ -465,6 +529,48 @@ clear_animation_timeout (NMApplet *applet)
 	}
 }
 
+static gboolean
+applet_is_any_device_activating (NMApplet *applet)
+{
+	const GPtrArray *devices;
+	int i;
+
+	/* Check for activating devices */
+	devices = nm_client_get_devices (applet->nm_client);
+	for (i = 0; devices && (i < devices->len); i++) {
+		NMDevice *candidate = NM_DEVICE (g_ptr_array_index (devices, i));
+		NMDeviceState state;
+
+		state = nm_device_get_state (candidate);
+		if (state > NM_DEVICE_STATE_DISCONNECTED && state < NM_DEVICE_STATE_ACTIVATED)
+			return TRUE;
+	}
+	return FALSE;
+}
+
+static gboolean
+applet_is_any_vpn_activating (NMApplet *applet)
+{
+	const GPtrArray *connections;
+	int i;
+
+	connections = nm_client_get_active_connections (applet->nm_client);
+	for (i = 0; connections && (i < connections->len); i++) {
+		NMActiveConnection *candidate = NM_ACTIVE_CONNECTION (g_ptr_array_index (connections, i));
+		NMVPNConnectionState vpn_state;
+
+		if (NM_IS_VPN_CONNECTION (candidate)) {
+			vpn_state = nm_vpn_connection_get_vpn_state (NM_VPN_CONNECTION (candidate));
+			if (   vpn_state == NM_VPN_CONNECTION_STATE_PREPARE
+			    || vpn_state == NM_VPN_CONNECTION_STATE_NEED_AUTH
+			    || vpn_state == NM_VPN_CONNECTION_STATE_CONNECT
+			    || vpn_state == NM_VPN_CONNECTION_STATE_IP_CONFIG_GET) {
+				return TRUE;
+			}
+		}
+	}
+	return FALSE;
+}
 static void
 update_connection_timestamp (NMActiveConnection *active,
                              NMConnection *connection,
@@ -562,58 +668,6 @@ make_vpn_disconnection_message (NMVPNConnection *vpn,
 	return g_strdup_printf (_("\nThe VPN connection '%s' disconnected."), nm_setting_connection_get_id (s_con));
 }
 
-typedef struct {
-	const char *tag;
-	const char *replacement;
-} Tag;
-
-static Tag banner_tags[] = {
-	{ "<center>", NULL },
-	{ "</center>", NULL },
-	{ "<p>", "\n" },
-	{ "</p>", NULL },
-	{ "<B>", "<b>" },
-	{ "</B>", "</b>" },
-	{ "<I>", "<i>" },
-	{ "</I>", "</i>" },
-	{ "<u>", "<u>" },
-	{ "</u>", "</u>" },
-	{ NULL, NULL }
-};
-
-static char *
-construct_vpn_banner (const char *src)
-{
-	const char *p = src;
-	GString *banner;
-
-	/* Filter the banner text and get rid of some HTML tags since the
-	 * notification spec only allows a subset of HTML.
-	 */
-
-	banner = g_string_sized_new (strlen (src) + 5);
-	g_string_append_c (banner, '\n');
-	while (*p) {
-		Tag *t = &banner_tags[0];
-		gboolean found = FALSE;
-
-		while (t->tag) {
-			if (strncasecmp (p, t->tag, strlen (t->tag)) == 0) {
-				p += strlen (t->tag);
-				if (t->replacement)
-					g_string_append (banner, t->replacement);
-				found = TRUE;
-				break;
-			}
-			t++;
-		}
-		if (!found)
-			g_string_append_c (banner, *p++);
-	}
-
-	return g_string_free (banner, FALSE);
-}
-
 static void
 vpn_connection_state_changed (NMVPNConnection *vpn,
                               NMVPNConnectionState state,
@@ -623,22 +677,27 @@ vpn_connection_state_changed (NMVPNConnection *vpn,
 	NMApplet *applet = NM_APPLET (user_data);
 	NMConnection *connection;
 	const char *banner;
-	char *title = NULL, *msg = NULL;
-	gboolean clear_timeout = TRUE;
+	char *title = NULL, *msg;
+	gboolean device_activating, vpn_activating;
+
+	device_activating = applet_is_any_device_activating (applet);
+	vpn_activating = applet_is_any_vpn_activating (applet);
 
 	switch (state) {
 	case NM_VPN_CONNECTION_STATE_PREPARE:
 	case NM_VPN_CONNECTION_STATE_NEED_AUTH:
 	case NM_VPN_CONNECTION_STATE_CONNECT:
 	case NM_VPN_CONNECTION_STATE_IP_CONFIG_GET:
-		start_animation_timeout (applet);
-		clear_timeout = FALSE;
+		/* Be sure to turn animation timeout on here since the dbus signals
+		 * for new active connections might not have come through yet.
+		 */
+		vpn_activating = TRUE;
 		break;
 	case NM_VPN_CONNECTION_STATE_ACTIVATED:
 		banner = nm_vpn_connection_get_banner (vpn);
 		if (banner && strlen (banner)) {
 			title = _("VPN Login Message");
-			msg = construct_vpn_banner (banner);
+			msg = g_strdup_printf ("%s\n", banner);
 			applet_do_notify (applet, NOTIFY_URGENCY_LOW, title, msg,
 			                  "gnome-lockscreen", NULL, NULL, NULL, NULL);
 			g_free (msg);
@@ -668,8 +727,11 @@ vpn_connection_state_changed (NMVPNConnection *vpn,
 		break;
 	}
 
-	if (clear_timeout)
+	if (device_activating || vpn_activating)
+		start_animation_timeout (applet);
+	else
 		clear_animation_timeout (applet);
+
 	applet_schedule_update_icon (applet);
 }
 
@@ -969,6 +1031,9 @@ applet_find_active_connection_for_device (NMDevice *device,
 		service_name = nm_active_connection_get_service_name (active);
 		connection_path = nm_active_connection_get_connection (active);
 
+		if (!devices || !service_name || !connection_path)
+			continue;
+
 		if (!nm_g_ptr_array_contains (devices, device))
 			continue;
 
@@ -976,8 +1041,12 @@ applet_find_active_connection_for_device (NMDevice *device,
 			NMDBusConnection *tmp;
 
 			tmp = nm_dbus_settings_get_connection_by_path (applet->dbus_settings, connection_path);
-			if (tmp)
+			if (tmp) {
 				connection = nm_exported_connection_get_connection (NM_EXPORTED_CONNECTION (tmp));
+				if (out_active)
+					*out_active = active;
+				break;
+			}
 		} else if (!strcmp (service_name, NM_DBUS_SERVICE_USER_SETTINGS)) {
 			NMAGConfConnection *tmp;
 
@@ -1237,6 +1306,30 @@ nma_set_networking_enabled_cb (GtkWidget *widget, NMApplet *applet)
 	nm_client_sleep (applet->nm_client, !state);
 }
 
+
+static void
+nma_set_notifications_enabled_cb (GtkWidget *widget, NMApplet *applet)
+{
+	gboolean state;
+
+	g_return_if_fail (applet != NULL);
+
+	state = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget));
+
+	gconf_client_set_bool (applet->gconf_client,
+	                       PREF_DISABLE_CONNECTED_NOTIFICATIONS,
+	                       !state,
+	                       NULL);
+	gconf_client_set_bool (applet->gconf_client,
+	                       PREF_DISABLE_DISCONNECTED_NOTIFICATIONS,
+	                       !state,
+	                       NULL);
+	gconf_client_set_bool (applet->gconf_client,
+	                       PREF_SUPPRESS_WIRELESS_NETWORKS_AVAILABLE,
+	                       !state,
+	                       NULL);
+}
+
 /*
  * nma_menu_show_cb
  *
@@ -1348,6 +1441,7 @@ nma_context_menu_update (NMApplet *applet)
 	NMState state;
 	gboolean have_wireless = FALSE;
 	gboolean wireless_hw_enabled;
+	gboolean notifications_enabled = TRUE;
 
 	state = nm_client_get_state (applet->nm_client);
 
@@ -1377,6 +1471,17 @@ nma_context_menu_update (NMApplet *applet)
 	gtk_widget_set_sensitive (GTK_WIDGET (applet->wifi_enabled_item),
 	                          wireless_hw_enabled);
 
+	/* Enabled notifications */
+	g_signal_handler_block (G_OBJECT (applet->notifications_enabled_item),
+	                        applet->notifications_enabled_toggled_id);
+	if (   gconf_client_get_bool (applet->gconf_client, PREF_DISABLE_CONNECTED_NOTIFICATIONS, NULL)
+	    && gconf_client_get_bool (applet->gconf_client, PREF_DISABLE_DISCONNECTED_NOTIFICATIONS, NULL)
+	    && gconf_client_get_bool (applet->gconf_client, PREF_SUPPRESS_WIRELESS_NETWORKS_AVAILABLE, NULL))
+		notifications_enabled = FALSE;
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (applet->notifications_enabled_item), notifications_enabled);
+	g_signal_handler_unblock (G_OBJECT (applet->notifications_enabled_item),
+	                          applet->notifications_enabled_toggled_id);
+
 	/* Don't show wifi-specific stuff if wireless is off */
 	if (state != NM_STATE_ASLEEP) {
 		const GPtrArray *devices;
@@ -1465,6 +1570,17 @@ static GtkWidget *nma_context_menu_create (NMApplet *applet)
 
 	nma_menu_add_separator_item (GTK_WIDGET (menu));
 
+	/* Toggle notifications item */
+	applet->notifications_enabled_item = gtk_check_menu_item_new_with_mnemonic (_("Enable N_otifications"));
+	id = g_signal_connect (applet->notifications_enabled_item,
+	                       "toggled",
+	                       G_CALLBACK (nma_set_notifications_enabled_cb),
+	                       applet);
+	applet->notifications_enabled_toggled_id = id;
+	gtk_menu_shell_append (menu, applet->notifications_enabled_item);
+
+	nma_menu_add_separator_item (GTK_WIDGET (menu));
+
 	/* 'Connection Information' item */
 	applet->info_menu_item = gtk_image_menu_item_new_with_mnemonic (_("Connection _Information"));
 	g_signal_connect_swapped (applet->info_menu_item,
@@ -1599,28 +1715,42 @@ applet_common_device_state_changed (NMDevice *device,
                                     NMDeviceStateReason reason,
                                     NMApplet *applet)
 {
+	gboolean device_activating = FALSE, vpn_activating = FALSE;
 	NMConnection *connection;
 	NMActiveConnection *active = NULL;
 
+	device_activating = applet_is_any_device_activating (applet);
+	vpn_activating = applet_is_any_vpn_activating (applet);
+
 	switch (new_state) {
 	case NM_DEVICE_STATE_PREPARE:
 	case NM_DEVICE_STATE_CONFIG:
 	case NM_DEVICE_STATE_NEED_AUTH:
 	case NM_DEVICE_STATE_IP_CONFIG:
-		start_animation_timeout (applet);
+		/* Be sure to turn animation timeout on here since the dbus signals
+		 * for new active connections or devices might not have come through yet.
+		 */
+		device_activating = TRUE;
 		break;
 	case NM_DEVICE_STATE_ACTIVATED:
 		/* If the device activation was successful, update the corresponding
 		 * connection object with a current timestamp.
 		 */
 		connection = applet_find_active_connection_for_device (device, applet, &active);
-		if (connection && active)
+		if (connection && (nm_connection_get_scope (connection) == NM_CONNECTION_SCOPE_USER))
 			update_connection_timestamp (active, connection, applet);
-		/* Fall through */
+		break;
 	default:
-		clear_animation_timeout (applet);
 		break;
 	}
+
+	/* If there's an activating device but we're not animating, start animation.
+	 * If we're animating, but there's no activating device or VPN, stop animating.
+	 */
+	if (device_activating || vpn_activating)
+		start_animation_timeout (applet);
+	else
+		clear_animation_timeout (applet);
 }
 
 static void
@@ -2470,6 +2600,14 @@ applet_pre_keyring_callback (gpointer user_data)
 	}
 }
 
+static void
+exit_cb (GObject *ignored, gpointer user_data)
+{
+	NMApplet *applet = user_data;
+
+	g_main_loop_quit (applet->loop);
+}
+
 static GObject *
 constructor (GType type,
              guint n_props,
@@ -2477,6 +2615,7 @@ constructor (GType type,
 {
 	NMApplet *applet;
 	AppletDBusManager *dbus_mgr;
+	GList *server_caps, *iter;
 
 	applet = NM_APPLET (G_OBJECT_CLASS (nma_parent_class)->constructor (type, n_props, construct_props));
 
@@ -2507,28 +2646,35 @@ constructor (GType type,
 	if (!notify_is_initted ())
 		notify_init ("NetworkManager");
 
+	server_caps = notify_get_server_caps();
+	applet->notify_with_actions = FALSE;
+	for (iter = server_caps; iter; iter = g_list_next (iter)) {
+		if (!strcmp ((const char *) iter->data, NOTIFY_CAPS_ACTIONS_KEY))
+			applet->notify_with_actions = TRUE;
+	}
+
+	g_list_foreach (server_caps, (GFunc) g_free, NULL);
+	g_list_free (server_caps);
+
 	dbus_mgr = applet_dbus_manager_get ();
 	if (dbus_mgr == NULL) {
 		nm_warning ("Couldn't initialize the D-Bus manager.");
 		g_object_unref (applet);
 		return NULL;
 	}
+	g_signal_connect (G_OBJECT (dbus_mgr), "exit-now", G_CALLBACK (exit_cb), applet);
 
 	applet->dbus_settings = (NMDBusSettings *) nm_dbus_settings_system_new (applet_dbus_manager_get_connection (dbus_mgr));
 
-	applet->gconf_settings = nma_gconf_settings_new ();
+	applet->gconf_settings = nma_gconf_settings_new (applet_dbus_manager_get_connection (dbus_mgr));
 	g_signal_connect (applet->gconf_settings, "new-secrets-requested",
 	                  G_CALLBACK (applet_settings_new_secrets_requested_cb),
 	                  applet);
 
-	dbus_g_connection_register_g_object (applet_dbus_manager_get_connection (dbus_mgr),
-	                                     NM_DBUS_PATH_SETTINGS,
-	                                     G_OBJECT (applet->gconf_settings));
-
 	/* Start our DBus service */
 	if (!applet_dbus_manager_start_service (dbus_mgr)) {
-			g_object_unref (applet);
-			return NULL;
+		g_object_unref (applet);
+		return NULL;
 	}
 
 	/* Initialize device classes */
@@ -2547,7 +2693,7 @@ constructor (GType type,
 	foo_client_setup (applet);
 
 	/* timeout to update connection timestamps every 5 minutes */
-	applet->update_timestamps_id = g_timeout_add (300000,
+	applet->update_timestamps_id = g_timeout_add_seconds (300,
 			(GSourceFunc) periodic_update_active_connection_timestamps, applet);
 
 	nm_gconf_set_pre_keyring_callback (applet_pre_keyring_callback, applet);
